@@ -2,89 +2,101 @@ package Net::Facebook::Oauth2;
 
 use strict;
 use warnings;
-our $VERSION = '0.02';
 use LWP::UserAgent;
 use URI::Escape;
 use JSON::Any;
+use Carp;
+
+our $VERSION = '0.05';
+
+use constant ACCESS_TOKEN_URL => 'https://graph.facebook.com/oauth/access_token';
+use constant AUTHORIZE_URL => 'https://www.facebook.com/dialog/oauth';
 
 sub new {
-    
     my ($class,%options) = @_;
-    
     my $self = {};
     $self->{options} = \%options;
+    
+    if (!$options{access_token}){
+        croak "Yuo must provide your application id when construct new method\n Net::Facebook::Oauth2->new( application_id => '...' )" unless defined $self->{options}->{application_id};
+        croak "Yuo must provide your application secret when construct new method\n Net::Facebook::Oauth2->new( application_secret => '...' )" unless defined $self->{options}->{application_secret};
+    }
+    
     $self->{browser} = LWP::UserAgent->new;
-    $self->{access_token_url} = $options{access_token_url} || "https://graph.facebook.com/oauth/access_token";
-    $self->{authorize_url} = $options{authorize_url} || "https://graph.facebook.com/oauth/authorize";
+    $self->{access_token_url} = $options{access_token_url} || ACCESS_TOKEN_URL;
+    $self->{authorize_url} = $options{authorize_url} || AUTHORIZE_URL;
     $self->{access_token} = $options{access_token};
     $self->{display} = $options{display} || 'page'; ##other values popup and wab
     return bless($self, $class);
-  
 }
 
 sub get_authorization_url {
     
     my ($self,%params) = @_;
     
-    $params{callback} = $self->{options}->{callback} unless defined $params{callback};
-    die "You must pass a callback parameter with Oauth v2.0" unless defined $params{callback};
+    $params{callback} ||= $self->{options}->{callback};
+    croak "You must pass a callback parameter with Oauth v2.0" unless defined $params{callback};
     
     $params{display} = $self->{display} unless defined $params{display};
-    
-    
     $self->{options}->{callback} = $params{callback};
     
     my $scope = join(",", @{$params{scope}}) if defined($params{scope});
     
-    my $url = $self->{authorize_url}."?client_id=".$self->{options}->{application_id}."&redirect_uri=".$params{callback};
-    $url = $url."&scope=$scope" if $scope;
-    $url = $url."&display=".$params{display};
+    my $url = $self->{authorize_url}
+    ."?client_id="
+    .uri_escape($self->{options}->{application_id})
+    ."&redirect_uri="
+    .uri_escape($params{callback});
+    
+    $url .= "&scope=$scope" if $scope;
+    $url .= "&display=".$params{display};
     
     return $url;
-    
 }
 
 
 sub get_access_token {
     
     my ($self,%params) = @_;
+    $params{callback} ||= $self->{options}->{callback};
+    $params{code} ||= $self->{options}->{code};
     
-    
-    $params{code} = $self->{options}->{code} unless defined $params{code};
-    die "You must pass a code parameter with Oauth v2.0" unless defined $params{code};
-    
+    croak "You must pass a code parameter with Oauth v2.0" unless defined $params{code};
+    croak "You must pass callback URL" unless defined $params{callback};
     $self->{options}->{code} = $params{code};
     
-    my $getURL = $self->{access_token_url}."?client_id=".$self->{options}->{application_id}."&redirect_uri=".$self->{options}->{callback}."&client_secret=".$self->{options}->{application_secret}."&code=$params{code}";
-
+    ###generating access token URL
+    my $getURL = $self->{access_token_url}
+    ."?client_id="
+    .uri_escape($self->{options}->{application_id})
+    ."&redirect_uri="
+    .uri_escape($params{callback})
+    ."&client_secret="
+    .uri_escape($self->{options}->{application_secret})
+    ."&code=$params{code}";
     
     my $response = $self->{browser}->get($getURL);
     
-    if ($response->{_rc} =~ /40/){
+    ##got an error response from facebook
+    ##die and display error message
+    if (!$response->is_success){
         my $j = JSON::Any->new;
         my $error = $j->jsonToObj($response->content());
-        die "'".$error->{error}->{type}."'"." ".$error->{error}->{message};
+        croak "'" .$error->{error}->{type}. "'" . " " .$error->{error}->{message};
     }
     
+    ##everything is ok proccess response and extract access token
     my $file = $response->content();
     my ($access_token,$expires) = split(/&/, $file);
     my ($string,$token) = split(/=/, $access_token);
     
-    #die 'Verifier Code is not valid' if !$token;
+    ###save access token
+    if ($token){
+        $self->{access_token} = $token;
+        return $token;
+    }
     
-    $self->{access_token} = $token if $token;
-    
-    return $token;
-    
-}
-
-
-
-sub _content {
-    
-    my ($self,$content) = @_;
-    $self->{content} = $content;
-    return $self;
+    croak "can't get access token";
     
 }
 
@@ -92,14 +104,11 @@ sub _content {
 sub get {
     
     my ($self,$url,$params) = @_;
-    
-    die "You must pass access_token" unless defined $self->{access_token};
+    croak "You must pass access_token" unless defined $self->{access_token};
     
     ##construct the new url
     my @array;
     
-    
-    
     while ( my ($key, $value) = each(%{$params})){
         $value = uri_escape($value);
         push(@array, "$key=$value");
@@ -107,108 +116,96 @@ sub get {
 
     my $string = join('&', @array);
     
-    $url = $url."?access_token=".$self->{access_token};
-    
-    $url = $url."&".$string if $string;
+    $url .= "?access_token="
+    .$self->{access_token};
+    $url .= "&".$string if $string;
     
     my $response = $self->{browser}->get($url);
-    my $file = $response->content();
-    return $self->_content($file);
-    
+    my $content = $response->content();
+    return $self->_content($content);
 }
 
-
 sub post {
-    
     my ($self,$url,$params) = @_;
-    
-    die "You must pass access_token" unless defined $self->{access_token};
-    
+    croak "You must pass access_token" unless defined $self->{access_token};
     $params->{access_token} = $self->{access_token};
-    
     my $response = $self->{browser}->post($url,$params);
-    my $file = $response->content();
-    return $self->_content($file);
-    
+    my $content = $response->content();
+    return $self->_content($content);
 }
 
 sub as_hash {
-    
     my ($self) = @_;
     my $j = JSON::Any->new;
     return $j->jsonToObj($self->{content});
-    
 }
 
-
-
 sub as_json {
-    
     my ($self) = @_;
     return $self->{content};
-    
+}
+
+sub _content {
+    my ($self,$content) = @_;
+    $self->{content} = $content;
+    return $self;
 }
 
 
 1;
 
-
 =head1 NAME
 
 Net::Facebook::Oauth2 - a simple Perl wrapper around Facebook OAuth v2.0 protocol
 
 =head1 SYNOPSIS
 
-use CGI;
-my $cgi = CGI->new;
-
-use Net::Facebook::Oauth2;
-
-my $fb = Net::Facebook::Oauth2->new(
-    application_id => 'your_application_id', 
-    application_secret => 'your_application_secret'
-);
-
-###get authorization URL for your application
-my $url = $fb->get_authorization_url(
-    scope => ['offline_access','publish_stream'],
-    callback => 'http://yourdomain.com/facebook/callback',
-    display => 'page'
-);
-
-####now redirect to this url
-print $cgi->redirect($url);
-
-
-##once user vauthorize your application facebook will send him/her back to your application
-##to the callback link provided above
-
-###in your callback block capture verifier code and get access_token
-
-my $fb = Net::Facebook::Oauth2->new(
-    application_id => 'your_application_id',
-    application_secret => 'your_application_secret'
-);
-
-my $access_token = $fb->get_access_token(code => $cgi->param('code'));
-###save this token in database or session
-
-##later on your application you can use this verifier code to comunicate
-##with facebook on behalf of this user
-
-my $fb = Net::Facebook::Oauth2->new(
-    access_token => $access_token
-);
-
-my $info = $fb->get(
-    'https://graph.facebook.com/me' ##Facebook API URL
-);
-
-
-print $info->as_json;
+    use CGI;
+    my $cgi = CGI->new;
     
+    use Net::Facebook::Oauth2;
     
+    my $fb = Net::Facebook::Oauth2->new(
+        application_id => 'your_application_id', 
+        application_secret => 'your_application_secret',
+        callback => 'http://yourdomain.com/facebook/callback'
+    );
     
+    ###get authorization URL for your application
+    my $url = $fb->get_authorization_url(
+        scope => ['offline_access','publish_stream'],
+        display => 'page'
+    );
+    
+    ####now redirect to this url
+    print $cgi->redirect($url);
+    
+    ##once user authorizes your application facebook will send him/her back to your application
+    ##to the callback link provided above
+    
+    ###in your callback block capture verifier code and get access_token
+    
+    my $fb = Net::Facebook::Oauth2->new(
+        application_id => 'your_application_id',
+        application_secret => 'your_application_secret',
+        callback => 'http://yourdomain.com/facebook/callback'
+    );
+    
+    my $access_token = $fb->get_access_token(code => $cgi->param('code'));
+    ###save this token in database or session
+    
+    ##later on your application you can use this verifier code to comunicate
+    ##with facebook on behalf of this user
+    
+    my $fb = Net::Facebook::Oauth2->new(
+        access_token => $access_token
+    );
+    
+    my $info = $fb->get(
+        'https://graph.facebook.com/me' ##Facebook API URL
+    );
+    
+    print $info->as_json;
 
 =head1 DESCRIPTION
 
@@ -221,10 +218,10 @@ For more information please see example folder shipped with this Module
 For more information about Facebook Oauth 2.0 API
 
 Please Check
-http://developers.facebook.com/docs/
+L<http://developers.facebook.com/docs/>
 
 get/post Facebook Graph API
-http://developers.facebook.com/docs/api
+L<http://developers.facebook.com/docs/api>
 
 =head1 USAGE
 
@@ -371,11 +368,10 @@ Mahmoud A. Mehyar, E<lt>mamod.mehyar@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2010 by Mahmoud A. Mehyar
+Copyright (C) 2012 by Mahmoud A. Mehyar
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
 at your option, any later version of Perl 5 you may have available.
 
-
 =cut
